@@ -13,230 +13,21 @@
 
 #include <string.h>
 
-#include "build-disjuncts.h"
-#include "dict-common.h"
+#include "dict-common/dict-affix.h"   // For is_stem()
+#include "dict-common/dict-common.h"
+#include "dict-common/dict-defines.h" // For SUBSCRIPT_MARK
+#include "dict-common/file-utils.h"
+#include "dict-common/idiom.h"
+#include "dict-common/regex-morph.h"
 #include "error.h"
-#include "print.h"
+#include "print/print.h"
 #include "externs.h"
-#include "idiom.h"
 #include "read-dict.h"
-#include "regex-morph.h"
 #include "string-set.h"
+#include "tokenize/tok-structures.h" // needed for MT_WALL
 #include "utilities.h"
 #include "word-file.h"
 
-/**
- * Format the given locale for use in setlocale().
- * POSIX systems and Windows use different conventions.
- * On Windows, convert to full language and territory names, because the
- * short ones doesn't work for some reason on every system (including MinGW).
- * @param dict Used for putting the returned value in a string-set.
- * @param ll Locale 2-letter language code.
- * @param cc Locale 2-letter territory code.
- * @return The formatted locale, directly usable in setlocale().
- */
-static const char * format_locale(Dictionary dict,
-                                  const char *ll, const char *cc)
-{
-	unsigned char *locale_ll = (unsigned char *)strdupa(ll);
-	unsigned char *locale_cc = (unsigned char *)strdupa(cc);
-
-	for (unsigned char *p = locale_ll; '\0' != *p; p++) *p = tolower(*p);
-	for (unsigned char *p = locale_cc; '\0' != *p; p++) *p = toupper(*p);
-
-#ifdef _WIN32
-	const int locale_size = strlen(ll) + 1 + strlen(cc) + 1;
-	char *locale = alloca(locale_size);
-	snprintf(locale, locale_size, "%s-%s", locale_ll, locale_cc);
-
-	wchar_t wlocale[LOCALE_NAME_MAX_LENGTH];
-	wchar_t wtmpbuf[LOCALE_NAME_MAX_LENGTH];
-	char tmpbuf[LOCALE_NAME_MAX_LENGTH];
-	char locale_buf[LOCALE_NAME_MAX_LENGTH];
-	size_t r;
-
-	r = mbstowcs(wlocale, locale, LOCALE_NAME_MAX_LENGTH);
-	if ((size_t)-1 == r)
-	{
-		prt_error("Error: Error converting %s to wide character.\n", locale);
-		return NULL;
-	}
-	wlocale[LOCALE_NAME_MAX_LENGTH-1] = L'\0';
-
-	if (0 >= GetLocaleInfoEx(wlocale, LOCALE_SENGLISHLANGUAGENAME,
-	                         wtmpbuf, LOCALE_NAME_MAX_LENGTH))
-	{
-		prt_error("Error: GetLocaleInfoEx LOCALE_SENGLISHLANGUAGENAME Locale=%s: \n"
-		          "Error %d", locale, (int)GetLastError());
-		return NULL;
-	}
-	r = wcstombs(tmpbuf, wtmpbuf, LOCALE_NAME_MAX_LENGTH);
-	if ((size_t)-1 == r)
-	{
-		prt_error("Error: Error converting locale language from wide character.\n");
-		return NULL;
-	}
-	tmpbuf[LOCALE_NAME_MAX_LENGTH-1] = '\0';
-	if (0 == strncmp(tmpbuf, "Unknown", 7))
-	{
-		prt_error("Error: Unknown territory code in locale \"%s\"\n", locale);
-		return NULL;
-	}
-	strcpy(locale_buf, tmpbuf);
-	strcat(locale_buf, "_");
-
-	if (0 >= GetLocaleInfoEx(wlocale, LOCALE_SENGLISHCOUNTRYNAME,
-	                         wtmpbuf, LOCALE_NAME_MAX_LENGTH))
-	{
-		prt_error("Error: GetLocaleInfoEx LOCALE_SENGLISHCOUNTRYNAME Locale=%s: \n"
-		          "Error %d", locale, (int)GetLastError());
-		return NULL;
-	}
-	r = wcstombs(tmpbuf, wtmpbuf, LOCALE_NAME_MAX_LENGTH);
-	if ((size_t)-1 == r)
-	{
-		prt_error("Error: Error converting locale territory from wide character.\n");
-		return NULL;
-	}
-	tmpbuf[LOCALE_NAME_MAX_LENGTH-1] = '\0';
-	if (0 == strncmp(tmpbuf, "Unknown", 7))
-	{
-		prt_error("Error: Unknown territory code in locale \"%s\"\n", locale);
-		return NULL;
-	}
-	locale = strcat(locale_buf, tmpbuf);
-#else /* Assuming POSIX */
-	const int locale_size = strlen(ll) + 1 + strlen(cc) + sizeof(".UTF-8");
-	char *locale = alloca(locale_size);
-	snprintf(locale, locale_size, "%s_%s.UTF-8", locale_ll, locale_cc);
-#endif
-
-	return string_set_add(locale, dict->string_set);
-}
-
-/**
- * Return a locale for the given dictionary, in the OS format.
- * - If <dictionary-locale> is defined, use it.
- * - Else use the locale from the environment.
- * - On Windows, if no environment locale use the default locale.
- *
- * <dictionary-locale>: LL4cc+;
- * LL is the ISO639 language code in uppercase,
- * cc is the ISO3166 territory code in lowercase.
- * This particular capitalization is needed for the value to be a
- * valid LG connector.
- * For transliterated dictionaries:
- * <dictionary-locale>: C+;
- *
- * @param dict The dictionary for which the locale is needed.
- * @return The locale, in a format suitable for use by setlocale().
- */
-const char * linkgrammar_get_dict_locale(Dictionary dict)
-{
-	if (dict->locale) return dict->locale;
-
-	const char *locale;
-	Dict_node *dn = lookup_list(dict, "<dictionary-locale>");
-
-	if (NULL == dn)
-	{
-		lgdebug(D_USER_FILES, "Debug: Dictionary '%s': Locale is not defined.\n",
-		        dict->name);
-		goto locale_error;
-	}
-
-	if (0 == strcmp(dn->exp->u.string, "C"))
-	{
-		locale = string_set_add("C", dict->string_set);
-	}
-	else
-	{
-		char c;
-		char locale_ll[4], locale_cc[3];
-		int locale_numelement = sscanf(dn->exp->u.string, "%3[A-Z]4%2[a-z]%c",
-										locale_ll, locale_cc, &c);
-		if (2 != locale_numelement)
-		{
-			prt_error("Error: \"<dictionary-locale>: %s\" "
-			          "should be in the form LL4cc+\n"
-						 "\t(LL: language code; cc: territory code) "
-						 "\tor C+ for transliterated dictionaries.\n",
-						 dn->exp->u.string);
-			goto locale_error;
-		}
-
-		locale = format_locale(dict, locale_ll, locale_cc);
-
-		if (!try_locale(locale))
-		{
-			prt_error("Debug: Dictionary \"%s\": Locale \"%s\" unknown\n",
-			          dict->name, locale);
-			goto locale_error;
-		}
-	}
-
-	free_lookup(dn);
-	lgdebug(D_USER_FILES, "Debug: Dictionary locale: \"%s\"\n", locale);
-	dict->locale = locale;
-	return locale;
-
-locale_error:
-	{
-		free_lookup(dn);
-
-		const char *locale = get_default_locale();
-		if (NULL == locale) return NULL;
-		const char *sslocale = string_set_add(locale, dict->string_set);
-		free((void *)locale);
-		prt_error("Info: Dictionary '%s': No locale definition - "
-		          "\"%s\" will be used.\n", dict->name, sslocale);
-		if (!try_locale(sslocale))
-		{
-			lgdebug(D_USER_FILES, "Debug: Unknown locale \"%s\"...\n", sslocale);
-			return NULL;
-		}
-		return sslocale;
-	}
-}
-
-const char * linkgrammar_get_version(void)
-{
-	const char *s = "link-grammar-" LINK_VERSION_STRING;
-	return s;
-}
-
-const char * linkgrammar_get_dict_version(Dictionary dict)
-{
-	char * ver;
-	char * p;
-	Dict_node *dn;
-	Exp *e;
-
-	if (dict->version) return dict->version;
-
-	/* The newer dictionaries should contain a macro of the form:
-	 * <dictionary-version-number>: V4v6v6+;
-	 * which would indicate dictionary version 4.6.6
-	 * Older dictionaries contain no version info.
-	 */
-	dn = lookup_list(dict, "<dictionary-version-number>");
-	if (NULL == dn) return "[unknown]";
-
-	e = dn->exp;
-	ver = strdup(&e->u.string[1]);
-	p = strchr(ver, 'v');
-	while (p)
-	{
-		*p = '.';
-		p = strchr(p+1, 'v');
-	}
-
-	free_lookup(dn);
-	dict->version = string_set_add(ver, dict->string_set);
-	free(ver);
-	return dict->version;
-}
-
 /*
   The dictionary format:
 
@@ -348,17 +139,15 @@ static void dict_error2(Dictionary dict, const char * s, const char *s2)
 
 	if (s2)
 	{
-		err_msg(lg_Error, "Error parsing dictionary %s.\n"
+		prt_error("Error parsing dictionary %s.\n"
 		          "%s %s\n\t line %d, tokens = %s",
-		        dict->name,
-		        s, s2, dict->line_number, tokens);
+		          dict->name, s, s2, dict->line_number, tokens);
 	}
 	else
 	{
-		err_msg(lg_Error, "Error parsing dictionary %s.\n"
+		prt_error("Error parsing dictionary %s.\n"
 		          "%s\n\t line %d, tokens = %s",
-		        dict->name,
-		        s, dict->line_number, tokens);
+		          dict->name, s, dict->line_number, tokens);
 	}
 	dict->recursive_error = false;
 }
@@ -370,7 +159,7 @@ static void dict_error(Dictionary dict, const char * s)
 
 static void warning(Dictionary dict, const char * s)
 {
-	err_msg(lg_Warn, "Warning: %s\n"
+	prt_error("Warning: %s\n"
 	        "\tline %d, current token = \"%s\"",
 	        s, dict->line_number, dict->token);
 }
@@ -863,7 +652,7 @@ rdictionary_lookup(Dict_node *llist,
 }
 
 /**
- * lookup_list() - return list of words in the file-backed dictionary.
+ * file_lookup_list() - return list of words in the file-backed dictionary.
  *
  * Returns a pointer to a lookup list of the words in the dictionary.
  * Matches include words that appear in idioms.  To exclude idioms, use
@@ -872,9 +661,9 @@ rdictionary_lookup(Dict_node *llist,
  * This list is made up of Dict_nodes, linked by their right pointers.
  * The node, file and string fields are copied from the dictionary.
  *
- * The returned list must be freed with free_lookup().
+ * The returned list must be freed with file_free_lookup().
  */
-Dict_node * lookup_list(const Dictionary dict, const char *s)
+Dict_node * file_lookup_list(const Dictionary dict, const char *s)
 {
 	Dict_node * llist =
 		rdictionary_lookup(NULL, dict->root, s, true, dict_order_bare);
@@ -882,15 +671,15 @@ Dict_node * lookup_list(const Dictionary dict, const char *s)
 	return llist;
 }
 
-bool boolean_lookup(Dictionary dict, const char *s)
+bool file_boolean_lookup(Dictionary dict, const char *s)
 {
-	Dict_node *llist = lookup_list(dict, s);
+	Dict_node *llist = file_lookup_list(dict, s);
 	bool boool = (llist != NULL);
-	free_lookup(llist);
+	file_free_lookup(llist);
 	return boool;
 }
 
-void free_lookup(Dict_node *llist)
+void file_free_lookup(Dict_node *llist)
 {
 	Dict_node * n;
 	while (llist != NULL)
@@ -912,7 +701,11 @@ void free_insert_list(Dict_node *ilist)
 	}
 }
 
-static Dict_node * dictionary_lookup_wild(Dictionary dict, const char *s)
+/**
+ * file_lookup_wild -- allows for wildcard searches (globs)
+ * Used to support the !! command in the parser command-line tool.
+ */
+Dict_node * file_lookup_wild(Dictionary dict, const char *s)
 {
 	bool lookup_idioms = test_enabled("lookup-idioms");
 	char * ds = strrchr(s, SUBSCRIPT_DOT); /* Only the rightmost dot is a
@@ -943,9 +736,9 @@ static Dict_node * dictionary_lookup_wild(Dictionary dict, const char *s)
  * This list is made up of Dict_nodes, linked by their right pointers.
  * The node, file and string fields are copied from the dictionary.
  *
- * The returned list must be freed with free_lookup().
+ * The returned list must be freed with file_free_lookup().
  */
-Dict_node * abridged_lookup_list(const Dictionary dict, const char *s)
+static Dict_node * abridged_lookup_list(const Dictionary dict, const char *s)
 {
 	Dict_node *llist;
 	llist = rdictionary_lookup(NULL, dict->root, s, false, dict_order_bare);
@@ -1108,14 +901,14 @@ static Exp * make_connector(Dictionary dict)
 		}
 		if (dn == NULL)
 		{
-			free_lookup(dn_head);
+			file_free_lookup(dn_head);
 			dict_error(dict, "\nPerhaps missing + or - in a connector.\n"
 			                 "Or perhaps you forgot the subscript on a word.\n"
 			                 "Or perhaps a word is used before it is defined.\n");
 			return NULL;
 		}
 		n = make_unary_node(&dict->exp_list, dn->exp);
-		free_lookup(dn_head);
+		file_free_lookup(dn_head);
 	}
 	else
 	{
@@ -1214,7 +1007,7 @@ void add_empty_word(Dictionary const dict, X_node *x)
 
 /**
  * Return true if the string is a (floating point) number.
- * Float points can be preceeded by a single plus or minus sign.
+ * Float points can be proceeded by a single plus or minus sign.
  */
 static bool is_number(const char * str)
 {
@@ -1700,7 +1493,7 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
 	}
 	else if (is_idiom_word(dn->string))
 	{
-		err_msg(lg_Warn, "Warning: Word \"%s\" found near line %d of %s.\n"
+		prt_error("Warning: Word \"%s\" found near line %d of %s.\n"
 		        "\tWords ending \".Ix\" (x a number) are reserved for idioms.\n"
 		        "\tThis word will be ignored.",
 		        dn->string, dict->line_number, dict->name);
@@ -1720,7 +1513,7 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
 			prt_error("\a\t%s", dnx->string);
 		}
 		prt_error("\a\n\tThis word will be ignored.\n");
-		free_lookup(dn_head);
+		file_free_lookup(dn_head);
 		free_dict_node(dn);
 	}
 	else
@@ -1933,160 +1726,3 @@ bool read_dictionary(Dictionary dict)
 }
 
 /* ======================================================================= */
-
-/**
- * Display the information about the given word.
- * If the word can split, display the information about each part.
- * Note that the splits may be invalid grammatically.
- *
- * Wild-card search is supported; the command-line user can type in !!word* or
- * !!word*.sub and get a list of all words that match up to the wild-card.
- * In this case no split is done.
- *
- * FIXME: Errors are printed twice, since display_word_split() is invoked twice
- * per word. One way to fix it is to change display_word_split() to return false
- * on failure. However, this is a big fix, because the failure is several
- * functions deep, all not returning a value or returning a value for another
- * purpose. An easy fix, which has advantages for other things, is to add (and
- * use here) a "char *last_error" field in the Dictionary structure, serving
- * like an "errno" of library calls.
- */
-
-static void display_word_split(Dictionary dict,
-                               const char * word,
-                               Parse_Options opts,
-                               void (*display)(Dictionary, const char *))
-{
-	Sentence sent;
-	struct Parse_Options_s display_word_opts = *opts;
-
-	if ('\0' == word) return; /* avoid trying null strings */
-
-	parse_options_set_spell_guess(&display_word_opts, 0);
-	sent = sentence_create(word, dict);
-	if (0 == sentence_split(sent, &display_word_opts))
-	{
-		/* List the splits */
-		print_sentence_word_alternatives(sent, false, NULL, NULL);
-		/* List the disjuncts information. */
-		print_sentence_word_alternatives(sent, false, display, NULL);
-	}
-	sentence_delete(sent);
-}
-
-#define DJ_COL_WIDTH sizeof("                         ")
-
-/**
- * Display the number of disjuncts associated with this dict node
- */
-static void display_counts(const char *word, Dict_node *dn)
-{
-	printf("matches:\n");
-
-	for (; dn != NULL; dn = dn->right)
-	{
-		unsigned int len;
-		char * s;
-		char * t;
-
-		len = count_disjunct_for_dict_node(dn);
-		s = strdup(dn->string);
-		t = strrchr(s, SUBSCRIPT_MARK);
-		if (t) *t = SUBSCRIPT_DOT;
-		printf("    ");
-		left_print_string(stdout, s, DJ_COL_WIDTH);
-		free(s);
-		printf(" %8u  disjuncts ", len);
-		if (dn->file != NULL)
-		{
-			printf("<%s>", dn->file->file);
-		}
-		printf("\n");
-	}
-}
-
-/**
- * Display the number of disjuncts associated with this dict node
- */
-static void display_expr(const char *word, Dict_node *dn)
-{
-	printf("expressions:\n");
-	for (; dn != NULL; dn = dn->right)
-	{
-		char * s;
-		char * t;
-
-		s = strdup(dn->string);
-		t = strrchr(s, SUBSCRIPT_MARK);
-		if (t) *t = SUBSCRIPT_DOT;
-		printf("    ");
-		left_print_string(stdout, s, DJ_COL_WIDTH);
-		free(s);
-		print_expression(dn->exp);
-		if (NULL != dn->right) /* avoid extra newlines at the end */
-			printf("\n\n");
-	}
-}
-
-static void display_word_info(Dictionary dict, const char * word)
-{
-	const char * regex_name;
-	Dict_node *dn_head;
-
-	dn_head = dictionary_lookup_wild(dict, word);
-	if (dn_head)
-	{
-		display_counts(word, dn_head);
-		free_lookup(dn_head);
-		return;
-	}
-
-	/* Recurse, if it's a regex match */
-	regex_name = match_regex(dict->regex_root, word);
-	if (regex_name)
-	{
-		display_word_info(dict, regex_name);
-		return;
-	}
-	printf("matches nothing in the dictionary.");
-}
-
-static void display_word_expr(Dictionary dict, const char * word)
-{
-	const char * regex_name;
-	Dict_node *dn_head;
-
-	dn_head = dictionary_lookup_wild(dict, word);
-	if (dn_head)
-	{
-		display_expr(word, dn_head);
-		free_lookup(dn_head);
-		return;
-	}
-
-	/* Recurse, if it's a regex match */
-	regex_name = match_regex(dict->regex_root, word);
-	if (regex_name)
-	{
-		display_word_expr(dict, regex_name);
-		return;
-	}
-	printf("matches nothing in the dictionary.");
-}
-
-/**
- *  dict_display_word_info() - display the information about the given word.
- */
-void dict_display_word_info(Dictionary dict, const char * word,
-		Parse_Options opts)
-{
-	display_word_split(dict, word, opts, display_word_info);
-}
-
-/**
- *  dict_display_word_expr() - display the connector info for a given word.
- */
-void dict_display_word_expr(Dictionary dict, const char * word, Parse_Options opts)
-{
-	display_word_split(dict, word, opts, display_word_expr);
-}
