@@ -11,7 +11,8 @@
 #include <string.h>
 #include "link-includes.h"
 #include "api-structures.h"
-#include "structures.h"
+#include "dict-common/dict-common.h"
+#include "dict-common/file-utils.h"
 #include "read-regex.h"
 
 /*
@@ -65,14 +66,14 @@ int read_regex_file(Dictionary dict, const char *file_name)
 		{
 			do
 			{
-				c = fgetc(fp);
+				c = lg_fgetc(fp);
 				if (c == '\n') { line++; }
 			}
 			while (lg_isspace(c));
 
 			if (c == '%')
 			{
-				while ((c != EOF) && (c != '\n')) { c = fgetc(fp); }
+				while ((c != EOF) && (c != '\n')) { c = lg_fgetc(fp); }
 				line++;
 			}
 		}
@@ -90,7 +91,7 @@ int read_regex_file(Dictionary dict, const char *file_name)
 				goto failure;
 			}
 			name[i++] = c;
-			c = fgetc(fp);
+			c = lg_fgetc(fp);
 		}
 		while ((!lg_isspace(c)) && (c != ':') && (c != EOF));
 		name[i] = '\0';
@@ -99,7 +100,7 @@ int read_regex_file(Dictionary dict, const char *file_name)
 		while (lg_isspace(c))
 		{
 			if (c == '\n') { line++; }
-			c = fgetc(fp);
+			c = lg_fgetc(fp);
 		}
 		if (c != ':')
 		{
@@ -111,7 +112,7 @@ int read_regex_file(Dictionary dict, const char *file_name)
 		do
 		{
 			if (c == '\n') { line++; }
-			c = fgetc(fp);
+			c = lg_fgetc(fp);
 		}
 		while (lg_isspace(c));
 		if (c == '!')
@@ -120,7 +121,7 @@ int read_regex_file(Dictionary dict, const char *file_name)
 			do
 			{
 				if (c == '\n') { line++; }
-				c = fgetc(fp);
+				c = lg_fgetc(fp);
 			}
 			while (lg_isspace(c));
 		}
@@ -141,7 +142,7 @@ int read_regex_file(Dictionary dict, const char *file_name)
 				goto failure;
 			}
 			prev = c;
-			c = fgetc(fp);
+			c = lg_fgetc(fp);
 			regex[i++] = c;
 		}
 		while ((c != '/' || prev == '\\') && (c != EOF));
