@@ -11,12 +11,34 @@
 /***************************************************************************/
 
 #include "error.h"
-#include "dict-common.h"
+#include "dict-common/dict-common.h"
+#include "dict-common/dict-defines.h" // for MAX_WORD
+#include "dict-common/file-utils.h"
 #include "string-set.h"
-#include "structures.h"
 #include "read-dict.h"
 #include "word-file.h"
 
+/** Replace the right-most dot with SUBSCRIPT_MARK */
+void patch_subscript(char * s)
+{
+	char *ds, *de;
+	int dp;
+	ds = strrchr(s, SUBSCRIPT_DOT);
+	if (!ds) return;
+
+	/* a dot at the end or a dot followed by a number is NOT
+	 * considered a subscript */
+	de = ds + 1;
+	if (*de == '\0') return;
+	dp = (int) *de;
+
+	/* If its followed by a UTF8 char, its NOT a subscript */
+	if (127 < dp || dp < 0) return;
+	/* assert ((0 < dp) && (dp <= 127), "Bad dictionary entry!"); */
+	if (isdigit(dp)) return;
+	*ds = SUBSCRIPT_MARK;
+}
+
 /**
  * Reads in one word from the file, allocates space for it,
  * and returns it.
@@ -42,7 +64,7 @@ static const char * get_a_word(Dictionary dict, FILE * fp)
 
 	if (j >= MAX_WORD) {
 		word[MAX_WORD] = '\0';
-		prt_error("The dictionary contains a word that is too long: %s", word);
+		prt_error("The dictionary contains a word that is too long: %s\n", word);
 		return ""; /* error indication */
 	}
 	word[j] = '\0';
