@@ -203,67 +203,37 @@ static void remap_linkages(Linkage lkg, const int *remap)
 	/* Unused memory not freed - all of it will be freed in free_linkages(). */
 }
 
-/**
- *  Print the chosen_disjuncts words.
- *  This is used for debug, e.g. for tracking them in the Wordgraph display.
- */
-static void print_chosen_disjuncts_words(const Linkage lkg)
-{
-	size_t i;
-	dyn_str *djwbuf = dyn_str_new();
-
-	err_msg(lg_Debug, "Linkage %p (%zu words): ", lkg, lkg->num_words);
-	for (i = 0; i < lkg->num_words; i++)
-	{
-		Disjunct *cdj = lkg->chosen_disjuncts[i];
-		const char *djw; /* disjunct word - the chosen word */
-
-		if (NULL == cdj)
-			djw = lkg->sent->word[i].optional ? "{}" : "[]";
-		else if ('\0' == cdj->word_string[0])
-			djw = "\\0"; /* null string - something is wrong */
-		else
-			djw = cdj->word_string;
-
-		dyn_strcat(djwbuf, djw);
-		dyn_strcat(djwbuf, " ");
-	}
-	err_msg(lg_Debug, "%s\n", djwbuf->str);
-	dyn_str_delete(djwbuf);
-}
-
 /**
  * Remove unlinked optional words from a linkage.
  * XXX Should we remove here also the dict-cap tokens? In any case, for now they
  * are left for debug.
  */
-#define D_REE 9
+#define D_REE 7
 void remove_empty_words(Linkage lkg)
 {
 	size_t i, j;
 	Disjunct **cdj = lkg->chosen_disjuncts;
 	int *remap = alloca(lkg->num_words * sizeof(*remap));
-
-	if (verbosity_level(+D_REE))
-	{
-		err_msg(lg_Debug, "chosen_disjuncts before:\n\\");
-		print_chosen_disjuncts_words(lkg);
-	}
+	Gword **wgp = lkg->wg_path;
 
 	for (i = 0, j = 0; i < lkg->num_words; i++)
 	{
-		if ((NULL == cdj[i]) && lkg->sent->word[i].optional)
+		/* Discard optional words that are not real null-words.  Note that
+		 * if optional words don't have non-optional words after them,
+		 * wg_path doesn't include them, and hence *wgp is NULL then. */
+		if ((NULL == *wgp) || ((*wgp)->sent_wordidx != i))
 		{
+			assert((NULL == cdj[i]) && lkg->sent->word[i].optional);
 			remap[i] = -1;
+			continue;
 		}
-		else
-		{
-			Disjunct *cdtmp = cdj[j];
-			cdj[j] = cdj[i];
-			cdj[i] = cdtmp; /* The SAT parser frees chosen_disjuncts elements. */
-			remap[i] = j;
-			j++;
-		}
+
+		Disjunct *cdtmp = cdj[j];
+		cdj[j] = cdj[i];
+		cdj[i] = cdtmp; /* The SAT parser frees chosen_disjuncts elements. */
+		remap[i] = j;
+		j++;
+		wgp++;
 	}
 	if (lkg->num_words != j)
 	{
@@ -271,12 +241,6 @@ void remove_empty_words(Linkage lkg)
 		lkg->num_words = j;
 		remap_linkages(lkg, remap); /* Update lkg->link_array and lkg->num_links. */
 	}
-
-	if (verbosity_level(+D_REE))
-	{
-		err_msg(lg_Debug, "chosen_disjuncts after:\n\\");
-		print_chosen_disjuncts_words(lkg);
-	}
 }
 #undef D_REE
 
@@ -330,8 +294,8 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 
 	memset(show_word, 0, linkage->num_words * sizeof(*show_word));
 
-	if (D_CCW <= opts->verbosity)
-		print_lwg_path(lwg_path);
+	if (verbosity_level(D_CCW))
+		print_lwg_path(lwg_path, "Linkage");
 
 	for (i = 0; i < linkage->num_words; i++)
 	{
@@ -460,11 +424,10 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 				/* Get rid of those ugly ".Ixx" */
 				if (is_idiom_word(t))
 				{
-					s = strdup(t);
+					s = strdupa(t);
 					sm = strrchr(s, SUBSCRIPT_MARK);
 					*sm = '\0';
 					t = string_set_add(s, sent->string_set);
-					free(s);
 				}
 				else if (HIDE_MORPHO)
 				{
@@ -497,7 +460,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 							join_alt = false;
 							break;
 						}
-						join_len += strlen(cdjp[j]->word_string);
+						join_len += strlen(cdjp[j]->word_string) + 1;
 						if ((*wgaltp)->morpheme_type & IS_REG_MORPHEME)
 							join_alt = true;
 					}
@@ -594,20 +557,21 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 			 */
 			if (t)
 			{
-				const char *sm = strrchr(t, SUBSCRIPT_MARK);
+
+				s = strdupa(t);
+				sm = strrchr(s, SUBSCRIPT_MARK);
+				if (sm) *sm = SUBSCRIPT_DOT;
 
 				if ((!(w->status & WS_GUESS) && (w->status & WS_INDICT))
 				    || !DISPLAY_GUESS_MARKS)
 				{
-					s = alloca(strlen(t)+1);
-					strcpy(s, t);
-					if (sm) s[sm-t] = SUBSCRIPT_DOT;
 					t = string_set_add(s, sent->string_set);
 				}
 				else
 				{
-					size_t baselen;
-					const char *regex_name;
+					const char *regex_name = w->regex_name;
+					/* 4 = 1(null) + 1(guess_mark) + 2 (sizeof "[]") */
+					int baselen = NULL == sm ? strlen(t) : (size_t)(sm-s);
 					char guess_mark = 0;
 
 					switch (w->status & WS_GUESS)
@@ -636,10 +600,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 					 * a guess indication but the last subword doesn't have, no guess
 					 * indication would be shown at all. */
 
-					regex_name = w->regex_name;
 					if ((NULL == regex_name) || HIDE_MORPHO) regex_name = "";
-					/* 4 = 1(null) + 1(guess_mark) + 2 (sizeof "[]") */
-					baselen = NULL == sm ? strlen(t) : (size_t)(sm-t);
 					s = alloca(strlen(t) + strlen(regex_name) + 4);
 					strncpy(s, t, baselen);
 					s[baselen] = '[';
@@ -647,9 +608,6 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 					strcpy(s + baselen + 2, regex_name);
 					strcat(s, "]");
 					if (NULL != sm) strcat(s, sm);
-					t = s;
-					sm = strrchr(t, SUBSCRIPT_MARK);
-					if (sm) s[sm-t] = SUBSCRIPT_DOT;
 					t = string_set_add(s, sent->string_set);
 				}
 			}
@@ -725,11 +683,8 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 
 	linkage->wg_path_display = n_lwg_path;
 
-	if (D_CCW <= opts->verbosity)
-	{
-		print_lwg_path(lwg_path);
-		print_lwg_path(n_lwg_path);
-	}
+	if (verbosity_level(D_CCW))
+		print_lwg_path(n_lwg_path, "Display");
 }
 #undef D_CCW
 
