@@ -193,7 +193,40 @@ static int num_optional_words(count_context_t *ctxt, int w1, int w2)
 	return n;
 }
 
+//#define DO_COUNT_TRACE
+
+#ifdef DO_COUNT_TRACE
+#define V(c) (!c?"(nil)":c->string)
+static Count_bin do_count1(int lineno, fast_matcher_t *mchxt,
+                          count_context_t *ctxt,
+                          int lw, int rw,
+                          Connector *le, Connector *re,
+                          int null_count);
+
+static Count_bin do_count(int lineno, fast_matcher_t *mchxt,
+                          count_context_t *ctxt,
+                          int lw, int rw,
+                          Connector *le, Connector *re,
+                          int null_count)
+{
+	static int level;
+
+	level++;
+	printf("%*sdo_count:%d lw=%d rw=%d le=%s re=%s null_count=%d\n",
+		    level*2, "", lineno, lw, rw, V(le), V(re), null_count);
+	Table_connector *t = find_table_pointer(ctxt, lw, rw, le, re, null_count);
+	Count_bin r = do_count1(lineno, mchxt, ctxt, lw, rw, le, re, null_count);
+	printf("%*sreturn%*s:%d=%lld\n", level*2, "", (!!t)*3, "(C)", lineno, r);
+	level--;
+
+	return r;
+}
+
+static Count_bin do_count1(int lineno, fast_matcher_t *mchxt,
+#define do_count(...) do_count(__LINE__, __VA_ARGS__)
+#else
 static Count_bin do_count(fast_matcher_t *mchxt,
+#endif
                           count_context_t *ctxt,
                           int lw, int rw,
                           Connector *le, Connector *re,
@@ -214,7 +247,12 @@ static Count_bin do_count(fast_matcher_t *mchxt,
 	 * This count must be updated before we return. */
 	t = table_store(ctxt, lw, rw, le, re, null_count);
 
-	if (rw == 1+lw)
+	int unparseable_len = rw-lw-1;
+
+#if 1
+	/* This check is not necessary for correctness, as it is handled in
+	 * the general case below. It looks like it should be slightly faster. */
+	if (unparseable_len == 0)
 	{
 		/* lw and rw are neighboring words */
 		/* You can't have a linkage here with null_count > 0 */
@@ -228,17 +266,25 @@ static Count_bin do_count(fast_matcher_t *mchxt,
 		}
 		return t->count;
 	}
+#endif
 
 	/* The left and right connectors are null, but the two words are
 	 * NOT next to each-other. */
 	if ((le == NULL) && (re == NULL))
 	{
-		if (!ctxt->islands_ok && (lw != -1))
+		int nopt_words = num_optional_words(ctxt, lw, rw);
+
+		if ((null_count == 0) || (!ctxt->islands_ok && (lw != -1)) )
 		{
-			/* If we don't allow islands (a set of words linked together
-			 * but separate from the rest of the sentence) then the
-			 * null_count of skipping n words is just n. */
-			if (null_count == (rw-lw-1) - num_optional_words(ctxt, lw, rw))
+			/* The null_count of skipping n words is just n.
+			 * In case the unparsable range contains optional words, we
+			 * don't know here how many of them are actually skipped, because
+			 * they may belong to different alternatives and essentially just
+			 * be ignored.  Hence the inequality - sane_linkage_morphism()
+			 * will discard the linkages with extra null words. */
+			if ((null_count <= unparseable_len) &&
+			    (null_count >= unparseable_len - nopt_words))
+
 			{
 				t->count = hist_one();
 			}
@@ -248,26 +294,20 @@ static Count_bin do_count(fast_matcher_t *mchxt,
 			}
 			return t->count;
 		}
-		if (null_count == 0)
+
+		/* Here null_count != 0 and we allow islands (a set of words
+		 * linked together but separate from the rest of the sentence).
+		 * Because we don't know here if an optional word is just
+		 * skipped or is a real null-word (see the comment above) we
+		 * try both possibilities: If a real null is encountered, the
+		 * rest of the sentence should contain one less null-word. Else
+		 * the rest of the sentence still contains the required */
+		t->count = zero;
+		w = lw + 1;
+		for (int opt = 0; opt <= !!ctxt->local_sent[w].optional; opt++)
 		{
-			/* There is no solution without nulls in this case. There is
-			 * a slight efficiency hack to separate this null_count==0
-			 * case out, but not necessary for correctness */
-			if ((rw-lw-1) == num_optional_words(ctxt, lw, rw))
-			{
-				t->count = hist_one();
-			}
-			else
-			{
-				t->count = zero;
-			}
-		}
-		else
-		{
-			t->count = zero;
-			Disjunct * d;
-			int w = lw + 1;
-			for (d = ctxt->local_sent[w].d; d != NULL; d = d->next)
+			null_count += opt;
+			for (Disjunct *d = ctxt->local_sent[w].d; d != NULL; d = d->next)
 			{
 				if (d->left == NULL)
 				{
