@@ -551,13 +551,12 @@ static bool alt_connection_possible(Connector *c1, Connector *c2,
  * actually matches lc or rc or both. The lw and rw are the words from
  * which lc and rc came respectively.
  *
- * The list is returned in a linked list of Match_nodes.  This list
+ * The list is returned in an array of Match_nodes.  This list
  * contains no duplicates, because when processing the ml list, only
  * elements whose match_left is true are included, and such elements are
  * not included again when processing the mr list.
  *
  * Note that if both lc and rc match the corresponding connectors of w,
-	gc.word = NULL;
  * match_left is set to true when the ml list is processed and the
  * disjunct is then added to the result list, and match_right of the
  * same disjunct is set to true when the mr list is processed, and this
