@@ -147,6 +147,17 @@ static void clean_up_string(char * s)
 	*y = '\0';
 }
 
+/**
+ * Prints string `s`, aligned to the left, in a field width `w`.
+ * If the width of `s` is shorter than `w`, then the remainder of
+ * field is padded with blanks (on the right).
+ */
+static void left_print_string(FILE * fp, const char * s, int w)
+{
+	int width = w + strlen(s) - utf8_strwidth(s);
+	fprintf(fp, "%-*s", width, s);
+}
+
 /**
  * Return TRUE if s points to a number:
  * optional + or - followed by 1 or more
@@ -293,8 +304,29 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 
 	if (s[0] == '!')
 	{
-		dict_display_word_info(dict, s+1, opts);
-		dict_display_word_expr(dict, s+1, opts);
+		char *out;
+
+		out = dict_display_word_info(dict, s+1, opts);
+		if (NULL != out)
+		{
+			printf("%s\n", out);
+		   free(out);
+			out = dict_display_word_expr(dict, s+1, opts);
+			if (NULL != out)
+			{
+				printf("%s", out);
+				free(out);
+			}
+			else
+			{
+				prt_error("Error: '%s': Internal Error: Missing expression.\n", s+1);
+			}
+		}
+		else
+		{
+			printf("Token \"%s\" matches nothing in the dictionary.\n", s+1);
+		}
+
 		return 0;
 	}
 #ifdef USE_REGEX_TOKENIZER
