@@ -186,6 +186,21 @@ static void process_linkage(Linkage linkage, Command_Options* copts)
 		fprintf(stdout, "%s", string);
 		linkage_free_diagram(string);
 	}
+	if ((mode = copts->display_constituents))
+	{
+		string = linkage_print_constituent_tree(linkage, mode);
+		if (string != NULL)
+		{
+			fprintf(stdout, "%s\n", string);
+			linkage_free_constituent_tree_str(string);
+		}
+		else
+		{
+			copts->display_constituents = 0;
+			fprintf(stderr, "Can't generate constituents.\n");
+			fprintf(stderr, "Constituent processing has been turned off.\n");
+		}
+	}
 	if (copts->display_links)
 	{
 		string = linkage_print_links_and_domains(linkage);
@@ -201,7 +216,7 @@ static void process_linkage(Linkage linkage, Command_Options* copts)
 	if (copts->display_disjuncts)
 	{
 		string = linkage_print_disjuncts(linkage);
-		fprintf(stdout, "%s", string);
+		fprintf(stdout, "%s\n", string);
 		linkage_free_disjuncts(string);
 	}
 	if (copts->display_postscript)
@@ -211,21 +226,6 @@ static void process_linkage(Linkage linkage, Command_Options* copts)
 		fprintf(stdout, "%s\n", string);
 		linkage_free_postscript(string);
 	}
-	if ((mode = copts->display_constituents))
-	{
-		string = linkage_print_constituent_tree(linkage, mode);
-		if (string != NULL)
-		{
-			fprintf(stdout, "%s\n", string);
-			linkage_free_constituent_tree_str(string);
-		}
-		else
-		{
-			copts->display_constituents = 0;
-			fprintf(stderr, "Can't generate constituents.\n");
-			fprintf(stderr, "Constituent processing has been turned off.\n");
-		}
-	}
 }
 
 static void print_parse_statistics(Sentence sent, Parse_Options opts)
@@ -266,10 +266,10 @@ static void print_parse_statistics(Sentence sent, Parse_Options opts)
  * the following can be used:
  * link-parser -limit=30000 -test=auto-next-linkage:20000 < file.batch
  */
-static int auto_next_linkage_test(const char *test)
+static int auto_next_linkage_test(const char *test_opt)
 {
 	char auto_next_linkage_str[] = ",auto-next-linkage";
-	char *auto_next_linkage_pos = strstr(test, auto_next_linkage_str);
+	char *auto_next_linkage_pos = strstr(test_opt, auto_next_linkage_str);
 	int max_display;
 
 	if (auto_next_linkage_pos == NULL) return 0;
@@ -425,29 +425,17 @@ static void batch_process_some_linkages(Label label,
 
 	if (there_was_an_error(label, sent, opts))
 	{
-		/* If linkages were found, print them.
-		 * The equality check takes care for the SAT solver, which, for
-		 * now, leaves num_linkages_found==0. In any case, if there are
-		 * actually no linkages, linkage_create() will return NULL, and
-		 * nothing will be processed. */
-		if (sentence_num_linkages_found(sent) >= 0) {
+		/* If we found at least one good linkage, print it. */
+		if (sentence_num_valid_linkages(sent) > 0) {
 			Linkage linkage = NULL;
-			/* If we found at least one good linkage, print it. */
-			if (sentence_num_valid_linkages(sent) > 0) {
-				int i;
-				for (i=0; i<sentence_num_linkages_post_processed(sent); i++)
-				{
-					if (0 == sentence_num_violations(sent, i))
-					{
-						linkage = linkage_create(i, sent, opts);
-						break;
-					}
-				}
-			}
-			else
+			int i;
+			for (i=0; i<sentence_num_linkages_post_processed(sent); i++)
 			{
-				/* This linkage will be bad; no good ones were found. */
-				linkage = linkage_create(0, sent, opts);
+				if (0 == sentence_num_violations(sent, i))
+				{
+					linkage = linkage_create(i, sent, opts);
+					break;
+				}
 			}
 			process_linkage(linkage, copts);
 			linkage_delete(linkage);
@@ -619,12 +607,12 @@ int main(int argc, char * argv[])
 	}
 
 	copts = command_options_create();
-	opts = copts->popts;
-	if (copts == NULL || opts == NULL || copts->panic_opts == NULL)
+	if (copts == NULL || copts->panic_opts == NULL)
 	{
 		prt_error("Fatal error: unable to create parse options\n");
 		exit(-1);
 	}
+	opts = copts->popts;
 
 	setup_panic_parse_options(copts->panic_opts);
 	copts->panic_mode = true;
